--[[
A generic button class

This code is MIT licensed, see http://www.opensource.org/licenses/mit-license.php
(C) 2010 - 2011 Gideros Mobile 
]]

ListValueButton = Core.class(Sprite)
function ListValueButton:init(upState_path, downState_path,listData,currentIndex)
	self.font = TTFont.new("edosz.ttf", 20,true)
	self.textColorUp = 0xFFFFFF;
	self.textColorDown = 0xBA0504;

	self.upState = Bitmap.new(Texture.new(upState_path),true)
	self.downState = Bitmap.new(Texture.new(downState_path),true)
	self.upState:setAnchorPoint(0.5,0.5)
	self.downState:setAnchorPoint(0.5,0.5)

	if listData then
		self.listData = listData;
		if currentIndex then
			self.currentIndex = currentIndex;
		else
			self.currentIndex = 1;
		end
		self.value = TextField.new(self.font,listData[self.currentIndex]);
		self.value:setTextColor( self.textColorUp )
		self.value:setAnchorPoint(0.5,0.5);
	end
	self.focus = false

	-- set the visual state as "up"
	self:updateVisualState(false)
	if self.value and (not self:contains(self.value)) then
		self:addChild(self.value)
	end

	-- register to all mouse and touch events
	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)

	self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
	self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
	self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
	self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end

function ListValueButton:changeNextValue()
	self.currentIndex = self.currentIndex + 1;
	if(self.currentIndex > #self.listData) then
		self.currentIndex = 1;
	end
	print ("Change next value to ",self.currentIndex);
	self.value:setText(self.listData[self.currentIndex]);
	self.value:setAnchorPoint(0.5,0.5);
end

function ListValueButton:onMouseDown(event)
	if self:hitTestPoint(event.x, event.y) then
		self.focus = true
		self:updateVisualState(true)
		event:stopPropagation()
	end
end

function ListValueButton:onMouseMove(event)
	if self.focus then
		if not self:hitTestPoint(event.x, event.y) then	
			self.focus = false
			self:updateVisualState(false)
		end
		event:stopPropagation()
	end
end

function ListValueButton:onMouseUp(event)
	if self.focus then
		self.focus = false
		self:updateVisualState(false)
		self:changeNextValue();
		self:dispatchEvent(Event.new("click"))	-- button is clicked, dispatch "click" event
		event:stopPropagation()
	end
end

-- if button is on focus, stop propagation of touch events
function ListValueButton:onTouchesBegin(event)
	if self.focus then
		event:stopPropagation()
	end
end

-- if button is on focus, stop propagation of touch events
function ListValueButton:onTouchesMove(event)
	if self.focus then
		event:stopPropagation()
	end
end

-- if button is on focus, stop propagation of touch events
function ListValueButton:onTouchesEnd(event)
	if self.focus then
		event:stopPropagation()
	end
end

-- if touches are cancelled, reset the state of the button
function ListValueButton:onTouchesCancel(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		event:stopPropagation()
	end
end

-- if state is true show downState else show upState
function ListValueButton:updateVisualState(state)
	if state then
		if self:contains(self.upState) then
			self:removeChild(self.upState)
			if self.value and (self:contains(self.value)) then
				self:removeChild(self.value)
			end			
		end
		
		if not self:contains(self.downState) then
			self:addChild(self.downState)
			if self.value and ( not self:contains(self.value)) then
				self:addChild(self.value)
				self.value:setTextColor( self.textColorDown)
			end			
		end
	else
		if self:contains(self.downState) then
			self:removeChild(self.downState)
			if self.value and (self:contains(self.value)) then
				self:removeChild(self.value)
			end			
		end
		
		if not self:contains(self.upState) then
			self:addChild(self.upState)
			if self.value and ( not self:contains(self.value)) then
				self:addChild(self.value)
				self.value:setTextColor( self.textColorUp)
			end			
		end
	end
end
